@@ -4,12 +4,14 @@ import re
 from pathlib import Path
 from typing import Any, Callable, Dict, Generic, List, Optional, Type, TypeVar, Union
 
+from pdm.pep517._vendor import toml
+from pdm.pep517._vendor.packaging.requirements import Requirement
+from pdm.pep517._vendor.packaging.specifiers import SpecifierSet
+from pdm.pep517._vendor.packaging.version import Version
 from pdm.pep517.legacy import convert_legacy
+from pdm.pep517.license import get_license_classifier, license_lookup
 from pdm.pep517.scm import get_version_from_scm
-
-from ._vendor import toml
-from ._vendor.packaging.requirements import Requirement
-from .utils import (
+from pdm.pep517.utils import (
     cd,
     ensure_pep440_req,
     find_packages_iter,
@@ -20,6 +22,16 @@ from .utils import (
 
 T = TypeVar("T")
 
+AVAILABLE_PYTHON_VERSIONS = (
+    "2.7",
+    "3.4",
+    "3.5",
+    "3.6",
+    "3.7",
+    "3.8",
+    "3.9",
+)
+
 
 class ProjectError(ValueError):
     pass
@@ -139,6 +151,8 @@ class Metadata:
     )
 
     def _get_license(self, value):
+        if not is_dict_like(value):
+            return ""
         if "file" in value and "text" in value:
             raise ProjectError(
                 "license table shouldn't specify both 'file' "
@@ -150,7 +164,15 @@ class Metadata:
             else value.get("text")
         )
 
+    def _get_license_type(self, value):
+        if is_dict_like(value):
+            if value.get("text", "") in license_lookup:
+                return value.get("text")
+        else:
+            return value
+
     license: MetaField[str] = MetaField("license", _get_license)
+    license_type: MetaField[str] = MetaField("license", _get_license_type)
 
     def _get_name(self, value):
         result = []
@@ -176,7 +198,28 @@ class Metadata:
     author_email: MetaField[str] = MetaField("authors", _get_email)
     maintainer: MetaField[str] = MetaField("maintainers", _get_name)
     maintainer_email: MetaField[str] = MetaField("maintainers", _get_email)
-    classifiers: MetaField[List[str]] = MetaField("classifiers")
+
+    @property
+    def classifiers(self):
+        classifers = set(self._metadata.get("classifiers", []))
+
+        if self.dynamic and "classifiers" in self.dynamic:
+            python_constraint = (
+                SpecifierSet(self.requires_python)
+                if self.requires_python
+                else SpecifierSet()
+            )
+
+            for version in AVAILABLE_PYTHON_VERSIONS:
+                if python_constraint.contains(Version(version)):
+                    classifers.add(f"Programming Language :: Python :: {version[0]}")
+                    classifers.add(f"Programming Language :: Python :: {version}")
+
+            if self.license_type:
+                classifers.add(get_license_classifier(self.license_type))
+
+        return sorted(classifers)
+
     keywords: MetaField[str] = MetaField("keywords")
     project_urls: MetaField[Dict[str, str]] = MetaField("urls")
     includes: MetaField[List[str]] = MetaField("includes")
@@ -199,7 +242,7 @@ class Metadata:
 
     @property
     def project_name(self) -> str:
-        return safe_name(self.name).lower()
+        return safe_name(self.name)
 
     @property
     def requires_extra(self) -> Dict[str, List[str]]:
@@ -241,23 +284,34 @@ class Metadata:
                 result[plugin] = [f"{k} = {v}" for k, v in value.items()]
         return result
 
+    @property
+    def package_dir(self) -> str:
+        """A directory that will be used to looking for packages."""
+        if "package-dir" in self._metadata:
+            return self._metadata["package-dir"]
+        elif self.filepath.parent.joinpath("src").is_dir():
+            return "src"
+        return ""
+
     def convert_package_paths(self) -> Dict[str, Union[List, Dict]]:
         """Return a {package_dir, packages, package_data, exclude_package_data} dict."""
-        package_dir = {}
         packages = []
         py_modules = []
         package_data = {"": ["*"]}
         exclude_package_data = {}
 
         with cd(self.filepath.parent.as_posix()):
+            src_dir = Path(self.package_dir or ".")
             if not self.includes:
-                if os.path.isdir("src"):
-                    package_dir[""] = "src"
-                    packages = list(find_packages_iter("src"))
-                else:
-                    packages = list(find_packages_iter(exclude=["tests", "tests.*"]))
+                packages = list(
+                    find_packages_iter(
+                        self.package_dir or ".",
+                        exclude=["tests", "tests.*"],
+                        src=src_dir,
+                    )
+                )
                 if not packages:
-                    py_modules = [path[:-3] for path in glob.glob("*.py")]
+                    py_modules = [path.name[:-3] for path in src_dir.glob("*.py")]
             else:
                 packages_set = set()
                 includes = self.includes
@@ -266,11 +320,11 @@ class Metadata:
                         include = include[:-2]
                     if "*" not in include and os.path.isdir(include):
                         dir_name = include.rstrip("/\\")
-                        temp = list(find_packages_iter(dir_name))
-                        if os.path.exists(dir_name + "/__init__.py"):
-                            temp = [dir_name] + [f"{dir_name}.{part}" for part in temp]
-                        elif temp:
-                            package_dir[""] = dir_name
+                        temp = list(
+                            find_packages_iter(dir_name, src=self.package_dir or ".")
+                        )
+                        if os.path.isfile(os.path.join(dir_name, "__init__.py")):
+                            temp.insert(0, dir_name)
                         packages_set.update(temp)
                         includes.remove(include)
                 packages[:] = list(packages_set)
@@ -295,7 +349,7 @@ class Metadata:
                     "Can't specify packages and py_modules at the same time."
                 )
         return {
-            "package_dir": package_dir,
+            "package_dir": {"": self.package_dir} if self.package_dir else {},
             "packages": packages,
             "py_modules": py_modules,
             "package_data": package_data,
