@@ -13,17 +13,26 @@ from io import StringIO
 from pathlib import Path
 from typing import List, Tuple, Union
 
-from ._vendor.packaging.markers import default_environment
-from ._vendor.packaging.specifiers import SpecifierSet
-from .base import Builder, BuildError
-from .utils import get_abi_tag, get_platform, safe_version, to_filename
+from pdm.pep517._vendor.packaging.markers import default_environment
+from pdm.pep517._vendor.packaging.specifiers import SpecifierSet
+from pdm.pep517.base import Builder, BuildError
+from pdm.pep517.utils import (
+    get_abi_tag,
+    get_package_version,
+    get_platform,
+    safe_version,
+    to_filename,
+)
 
-WHEEL_FILE_FORMAT = """\
+WHEEL_FILE_FORMAT = (
+    """\
 Wheel-Version: 1.0
-Generator: poetry {version}
+Generator: pdm-pep517 %s
 Root-Is-Purelib: {pure_lib}
 Tag: {tag}
 """
+    % get_package_version()
+)
 
 
 class WheelBuilder(Builder):
@@ -82,9 +91,7 @@ class WheelBuilder(Builder):
                 else info["python_version"].replace(".", "")
             )
             impl = impl_name + impl_ver
-            abi_tag = get_abi_tag(
-                tuple(int(p) for p in info["python_version"].split("."))
-            )
+            abi_tag = get_abi_tag()
             tag = (impl, abi_tag, platform)
         else:
             platform = "any"
@@ -157,12 +164,12 @@ class WheelBuilder(Builder):
             setup_py,
             "build",
             "-b",
-            str(self.project.root / "build"),
+            str(self.location / "build"),
         ]
-        proc = subprocess.run(build_args, capture_output=True)
-        print(proc.stdout)
-        if proc.returncode:
-            raise BuildError(f"Error occurs when running {build_args}:\n{proc.stderr}")
+        try:
+            subprocess.check_call(build_args)
+        except subprocess.CalledProcessError as e:
+            raise BuildError(f"Error occurs when running {build_args}:\n{e}")
         build_dir = self.location / "build"
         lib_dir = next(build_dir.glob("lib.*"), None)
         if not lib_dir:
@@ -224,7 +231,6 @@ class WheelBuilder(Builder):
     def _write_wheel_file(self, fp):
         fp.write(
             WHEEL_FILE_FORMAT.format(
-                version=self.meta.version,
                 pure_lib=self.meta.build is None,
                 tag=self.tag,
             )
